﻿using Microsoft.OpenApi.Extensions;
using ServiceCenter.Extensions;
using ServiceCenter.Logs;
using ServiceCenter.SqlSugars;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WCS.Core;
using WCS.Entity;
using WCS.Entity.Protocol.Station;
using WCS.WorkEngineering.Extensions;
using WCS.WorkEngineering.WebApi.Controllers;
using WCS.WorkEngineering.Worlds;
using DeviceFlags = WCS.WorkEngineering.Extensions.DeviceFlags;
using TaskStatus = WCS.Entity.TaskStatus;

namespace WCS.WorkEngineering.Systems.重绕区
{
    /// <summary>
    ///  组盘缓存生成AGV任务
    /// </summary>
    [BelongTo(typeof(RewindWorld))]
    [Description("组盘缓存生成AGV任务")]
    public class 组盘缓存生成AGV任务 : DeviceSystem<Device<IStation520, IStation521, IStation523>>
    {
        protected override bool ParallelDo => true;

        public override void Do(Device<IStation520, IStation521, IStation523> obj)
        {
            if (!obj.Data3.Status.HasFlag(StationStatus.Auto))
            {
                World.Log($"设备处于{obj.Data3.Status.GetDisplayName()}状态");
                return;
            }

            if (!obj.Data3.Status.HasFlag(StationStatus.PH_Status))
            {
                World.Log($"设备无光电");
                return;
            }

            if (obj.Data3.Status.HasFlag(StationStatus.PH_Status) && !obj.Data3.Status.HasFlag(StationStatus.Low_Station_2))
            {
                World.Log($"设备有光电，但不在低位", LogLevelEnum.High);
                return;
            }

            if (!obj.Data3.Status.HasFlag(StationStatus.Low_Station_2))
            {
                World.Log($"设备不在低位", LogLevelEnum.High);
                return;
            }

            SqlSugarHelper.Do(_db =>
            {
                var db = _db.Default;
                var task = db.Queryable<WCS_TaskInfo>().ReadPastUpdLock().Single(x => x.Uploaded == TaskStatus.ConveyorExecution && x.Status == TaskStatus.ConveyorExecution && x.ID == obj.Data.TaskNumber);
                if (task == null)
                {
                    World.Log($"未找到对应任务[{obj.Data.TaskNumber}],请检查该任务状态与更新状态是否均为输送机执行中", LogLevelEnum.High);
                    return;
                }
                var agvId = DateTime.Now.GetFormat(GetFormatterEnum.only) + task.ID;

                //开始创建AGV任务
                var agvTask = db.Queryable<WCS_AgvTaskInfo>().UpdLock().SplitTable(x => x.Take(2)).Single(x => x.TaskId == task.ID && x.TaskType == AGVTaskType.EnterDepot);
                if (agvTask == null)
                {                    
                    db.InsertableRowLock(new WCS_AgvTaskInfo()
                    {
                        ID = _db.GetAgvTaskId(),
                        TaskType = AGVTaskType.EnterDepot,
                        Status = AGVTaskStatus.NewBuild,
                        TaskId = task.ID,
                        Position = obj.Entity.Code,
                        Station = task.SrmStation,
                        WorkShop = 111,
                        AddWho = "WCS",
                        AddTime = DateTime.Now,
                        AgvID = agvId
                    }).SplitTable().ExecuteCommand();
                    World.Log($"{task.ID}未找到对应的AGV任务");
                    return;
                }

                if (agvTask.Status != AGVTaskStatus.NewBuild)
                {
                    World.Log($"AGV任务{agvTask.ID}状态不是新建", LogLevelEnum.High);
                    return;
                }
                //分配缓存货位 暂时先根据光电判断
                List<string> cacheWells = new List<string> { "9201", "9202", "9203", "9204", "9205", "9206", "9207", "9208", "9209" };
                var devs = Device.All.Where(x => cacheWells.Contains(x.Code)).Select(x =>
                                                    new Device<IStation520, IStation521, IStation523>(x, World));
                if (!devs.Any()) return;
                string sta = "";
                foreach ( var dev in devs ) 
                {
                    if (dev.Data3.Status.HasFlag(StationStatus.PH_Status) && !db.Queryable<WCS_AgvTaskInfo>().UpdLock().SplitTable(x => x.Take(2)).Any(x => x.Station == dev.Entity.Code && x.TaskType == AGVTaskType.EnterDepot 
                    && x.Status < AGVTaskStatus.MissionCompleted))
                    { 
                        sta = dev.Entity.Code;
                        break;
                    }
                }
                agvId = agvTask.AgvID;
   
                agvTask.Status = AGVTaskStatus.Confirm;
                agvTask.AgvStatus = AGVTaskStatus.Confirm;
                agvTask.Station = sta;
                db.UpdateableRowLock(agvTask).UpdateColumns(x => new { x.AgvID, x.Status, x.AgvStatus, x.Station}).SplitTable(x => x.Take(2)).ExecuteCommand();
                task.Status = TaskStatus.AGVExecution;
                task.AgvTaskID = agvTask.ID;
                task.EditTime = DateTime.Now;
                task.EditWho = "WCS";
                db.Updateable(task).UpdateColumns(x => new { x.Status, x.AgvTaskID, x.EditTime, x.EditWho }).ExecuteCommand();
                task.AddWCS_TASK_DTL(db, obj.Entity.Code, "AGV", $"任务下发至AGV{agvId}");
                var res = AgvApi.组盘入库(obj.Entity.Code, agvTask.Station, agvId);
            });
        }

       

        public override bool Select(Device dev)
        {
            return dev.Code is "9126" or "9137" or "9148";
        }
    }
}

